import uuid
import hashlib

from imghdr import what
from os.path import exists

import image_examples
from tree import *
from exceptions import GPerror

def catcher(func):
    def _wrapper(*args, **kwargs):
        try:
            func(*args, **kwargs)
        except AssertionError as error:
            raise GPerror(error.args)
    return _wrapper

class Image:
    def __init__(self, path: str) -> None:
        self.triggered = False

        @catcher
        def check():
            assert(exists(path), 'Файла с таким именем не существует.')
            assert(what(path) is not None, 'Файл не является картинкой.')
        check()

        with open(path, mode='rb') as file:
            self.data = file.read()
    
    def trigger(self):
        self.triggered = not self.triggered

    def is_triggered(self):
        return self.trigger


class Group_Image:
    def __init__(self) -> None:
        self.images = []
        self.root = None
        self.counter = 0
        self.tree_size = 0
        self.associate = dict()
        self.index_in_tree = []

    def __getitem__(self, index):
        return self.images[index]
    
    def trigger(self, index):
        # get the index in self.images

        @catcher
        def check():
            assert(0 <= index < len(self.images), 
                   'Индекс картинки должен быть положительным числом, меньшим, чем количество всех картинок в группе.')
        check()
        
        self.images[index].trigger()
        if self.images[index].is_triggered():
            self.root = insert(self.root, self.counter)
            self.index_in_tree[index] = self.counter
            self.associate[self.counter] = index
            self.counter += 1
            self.tree_size += 1
        else:
            self.root = deleteNode(self.root, self.index_in_tree[index])
            self.index_in_tree[index] = -1
            self.tree_size -= 1
            
    def add_to_group(self, picture: Image):
        @catcher
        def check():
            assert(isinstance(picture, Image), 'В группу можно добавить только объекты картинок.')
            assert(self.tree_size > 3, 'Слишком мало выбранных картинок. Высока вероятность отгадать пароль.')
        check()

        self.images.append(picture)
        self.index_in_tree.append(-1)
        
    def get_password(self):
        @catcher
        def check():
            assert(len(self.images) > 0, 'В группе нет картинок.')
            assert(self.tree_size > 0, 'В группе нет выбранных картинок.')
        check()

        def inorder(root):
            if root is None:
                return ''
            return inorder(root.left) + str(self.images[self.associate[root.key]].data) + inorder(root.right)
        password = inorder(self.root)

        salt = uuid.uuid4().hex
        return hashlib.sha256(salt.encode() + password.encode()).hexdigest() + ':' + salt
    
def check_password(hashed_password, user_password):
    password, salt = hashed_password.split(':')
    return password == hashlib.sha256(salt.encode() + user_password.encode()).hexdigest()


def get_link(name: str, number: int):
    """ Give the link to example picture from library repository. Choose the group: cats, vegetables or butterflies. Then choose the number of picture."""
    
    return f'https://raw.githubusercontent.com/macarell228/graphical_password/main/examples/{name}/{number}.avif'


# Операции с двоичным деревом поиска на Python

# Создаем узел
class Node:
    def __init__(self, key):
        self.key = key
        self.left = None
        self.right = None

# Отсортированный (inorder) обход
def inorder(root, level):
    if root is not None:

        # Обходим левое поддерево
        inorder(root.left, level + 1)

        # Обходим корень
        print(f'{root.key}:{level}', end=' ')

        # Обходим правое поддерево
        inorder(root.right, level + 1)

# Вставка элемента
def insert(node, key):
    # Возвращаем новый узел, если дерево пустое
    if node is None:
        return Node(key)

    # Идем в нужное место и вставляет узел
    if key < node.key:
        node.left = insert(node.left, key)
    else:
        node.right = insert(node.right, key)
    return node

# Поиск inorder-преемника
def minValueNode(node):
    current = node
    # Найдем крайний левый лист — он и будет inorder-преемником
    while(current.left is not None):
        current = current.left
    return current

# Удаление узла
def deleteNode(root, key):
    # Возвращаем, если дерево пустое
    if root is None:
        return root

    # Найдем узел, который нужно удалить
    if key < root.key:
        root.left = deleteNode(root.left, key)
    elif(key > root.key):
        root.right = deleteNode(root.right, key)
    else:
        # Если у узла только один дочерний узел или вообще их нет        
        if root.left is None:
            temp = root.right
            root = None
            return temp

        elif root.right is None:
            temp = root.left
            root = None
            return temp

        # Если у узла два дочерних узла,
        # помещаем центрированного преемника
        # на место узла, который нужно удалить
        temp = minValueNode(root.right)

        root.key = temp.key

        # Удаляем inorder-преемниа
        root.right = deleteNode(root.right, temp.key)

    return root


from setuptools import setup

# read the contents of your README file
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text(encoding='utf-8')

setup(name='graphical_password',
      version='2.0',
      description='Simple authorization creation based on a sequence of images',
      long_description=long_description,
      long_description_content_type='text/markdown',
      packages=['graphical_password'],
      author_email='egubareva06@mail.ru',
      zip_safe=False)